@@ -18,35 +18,38 @@
 
 import logging
 import socket
-from collections import namedtuple
 
-import socks
+try:
+    import socks
+except ImportError as e:
+    e.msg = (
+        "PySocks is missing and Pyrogram can't run without. "
+        "Please install it using \"pip3 install pysocks\"."
+    )
+
+    raise e
 
 log = logging.getLogger(__name__)
 
-Proxy = namedtuple("Proxy", ["enabled", "hostname", "port", "username", "password"])
-
 
 class TCP(socks.socksocket):
-    def __init__(self, proxy: Proxy):
+    def __init__(self, proxy: dict):
         super().__init__()
         self.settimeout(10)
-        self.proxy_enabled = False
-
-        if proxy and proxy.enabled:
-            self.proxy_enabled = True
+        self.proxy_enabled = proxy.get("enabled", False)
 
+        if proxy and self.proxy_enabled:
             self.set_proxy(
                 proxy_type=socks.SOCKS5,
-                addr=proxy.hostname,
-                port=proxy.port,
-                username=proxy.username,
-                password=proxy.password
+                addr=proxy.get("hostname", None),
+                port=proxy.get("port", None),
+                username=proxy.get("username", None),
+                password=proxy.get("password", None)
             )
 
             log.info("Using proxy {}:{}".format(
-                proxy.hostname,
-                proxy.port
+                proxy.get("hostname", None),
+                proxy.get("port", None)
             ))
 
     def close(self):
