@@ -17,7 +17,6 @@
 # along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.
 
 import logging
-import platform
 import threading
 import time
 from datetime import timedelta, datetime
@@ -47,19 +46,6 @@ class Result:
 
 
 class Session:
-    VERSION = __version__
-    APP_VERSION = "Pyrogram \U0001f525 {}".format(VERSION)
-
-    DEVICE_MODEL = "{} {}".format(
-        platform.python_implementation(),
-        platform.python_version()
-    )
-
-    SYSTEM_VERSION = "{} {}".format(
-        platform.system(),
-        platform.release()
-    )
-
     INITIAL_SALT = 0x616e67656c696361
     NET_WORKERS = 1
     WAIT_TIMEOUT = 15
@@ -84,24 +70,24 @@ class Session:
     }
 
     def __init__(self,
+                 client: pyrogram,
                  dc_id: int,
-                 test_mode: bool,
-                 proxy: dict,
                  auth_key: bytes,
-                 api_id: int,
-                 is_cdn: bool = False,
-                 client: pyrogram = None):
+                 is_media: bool = False,
+                 is_cdn: bool = False):
         if not Session.notice_displayed:
             print("Pyrogram v{}, {}".format(__version__, __copyright__))
             print("Licensed under the terms of the " + __license__, end="\n\n")
             Session.notice_displayed = True
 
-        self.connection = Connection(DataCenter(dc_id, test_mode), proxy)
-        self.api_id = api_id
-        self.is_cdn = is_cdn
         self.client = client
-
+        self.dc_id = dc_id
         self.auth_key = auth_key
+        self.is_media = is_media
+        self.is_cdn = is_cdn
+
+        self.connection = None
+
         self.auth_key_id = sha1(auth_key).digest()[-8:]
 
         self.session_id = Long(MsgId())
@@ -120,15 +106,26 @@ class Session:
         self.next_salt_thread = None
         self.next_salt_thread_event = Event()
 
+        self.net_worker_list = []
+
         self.is_connected = Event()
 
     def start(self):
         while True:
+            self.connection = Connection(DataCenter(self.dc_id, self.client.test_mode), self.client.proxy)
+
             try:
                 self.connection.connect()
 
                 for i in range(self.NET_WORKERS):
-                    Thread(target=self.net_worker, name="NetWorker#{}".format(i + 1)).start()
+                    self.net_worker_list.append(
+                        Thread(
+                            target=self.net_worker,
+                            name="NetWorker#{}".format(i + 1)
+                        )
+                    )
+
+                    self.net_worker_list[-1].start()
 
                 Thread(target=self.recv, name="RecvThread").start()
 
@@ -144,12 +141,14 @@ class Session:
                         functions.InvokeWithLayer(
                             layer,
                             functions.InitConnection(
-                                self.api_id,
-                                self.DEVICE_MODEL,
-                                self.SYSTEM_VERSION,
-                                self.APP_VERSION,
-                                "en", "", "en",
-                                functions.help.GetConfig(),
+                                api_id=self.client.api_id,
+                                app_version=self.client.app_version,
+                                device_model=self.client.device_model,
+                                system_version=self.client.system_version,
+                                system_lang_code=self.client.lang_code,
+                                lang_code=self.client.lang_code,
+                                lang_pack="",
+                                query=functions.help.GetConfig(),
                             )
                         )
                     )
@@ -190,9 +189,20 @@ class Session:
         for i in range(self.NET_WORKERS):
             self.recv_queue.put(None)
 
+        for i in self.net_worker_list:
+            i.join()
+
+        self.net_worker_list.clear()
+
         for i in self.results.values():
             i.event.set()
 
+        if not self.is_media and callable(self.client.disconnect_handler):
+            try:
+                self.client.disconnect_handler(self.client)
+            except Exception as e:
+                log.error(e, exc_info=True)
+
         log.debug("Session stopped")
 
     def restart(self):
@@ -208,14 +218,14 @@ class Session:
         msg_key = msg_key_large[8:24]
         aes_key, aes_iv = KDF(self.auth_key, msg_key, True)
 
-        return self.auth_key_id + msg_key + AES.ige_encrypt(data + padding, aes_key, aes_iv)
+        return self.auth_key_id + msg_key + AES.ige256_encrypt(data + padding, aes_key, aes_iv)
 
     def unpack(self, b: BytesIO) -> Message:
         assert b.read(8) == self.auth_key_id, b.getvalue()
 
         msg_key = b.read(16)
         aes_key, aes_iv = KDF(self.auth_key, msg_key, False)
-        data = BytesIO(AES.ige_decrypt(b.read(), aes_key, aes_iv))
+        data = BytesIO(AES.ige256_decrypt(b.read(), aes_key, aes_iv))
         data.read(8)
 
         # https://core.telegram.org/mtproto/security_guidelines#checking-session-id
@@ -245,63 +255,60 @@ class Session:
                 break
 
             try:
-                self.unpack_dispatch_and_ack(packet)
+                data = self.unpack(BytesIO(packet))
+
+                messages = (
+                    data.body.messages
+                    if isinstance(data.body, MsgContainer)
+                    else [data]
+                )
+
+                log.debug(data)
+
+                for msg in messages:
+                    if msg.seq_no % 2 != 0:
+                        if msg.msg_id in self.pending_acks:
+                            continue
+                        else:
+                            self.pending_acks.add(msg.msg_id)
+
+                    if isinstance(msg.body, (types.MsgDetailedInfo, types.MsgNewDetailedInfo)):
+                        self.pending_acks.add(msg.body.answer_msg_id)
+                        continue
+
+                    if isinstance(msg.body, types.NewSessionCreated):
+                        continue
+
+                    msg_id = None
+
+                    if isinstance(msg.body, (types.BadMsgNotification, types.BadServerSalt)):
+                        msg_id = msg.body.bad_msg_id
+                    elif isinstance(msg.body, (core.FutureSalts, types.RpcResult)):
+                        msg_id = msg.body.req_msg_id
+                    elif isinstance(msg.body, types.Pong):
+                        msg_id = msg.body.msg_id
+                    else:
+                        if self.client is not None:
+                            self.client.updates_queue.put(msg.body)
+
+                    if msg_id in self.results:
+                        self.results[msg_id].value = getattr(msg.body, "result", msg.body)
+                        self.results[msg_id].event.set()
+
+                if len(self.pending_acks) >= self.ACKS_THRESHOLD:
+                    log.info("Send {} acks".format(len(self.pending_acks)))
+
+                    try:
+                        self._send(types.MsgsAck(list(self.pending_acks)), False)
+                    except (OSError, TimeoutError):
+                        pass
+                    else:
+                        self.pending_acks.clear()
             except Exception as e:
                 log.error(e, exc_info=True)
 
         log.debug("{} stopped".format(name))
 
-    def unpack_dispatch_and_ack(self, packet: bytes):
-        data = self.unpack(BytesIO(packet))
-
-        messages = (
-            data.body.messages
-            if isinstance(data.body, MsgContainer)
-            else [data]
-        )
-
-        log.debug(data)
-
-        for msg in messages:
-            if msg.seq_no % 2 != 0:
-                if msg.msg_id in self.pending_acks:
-                    continue
-                else:
-                    self.pending_acks.add(msg.msg_id)
-
-            if isinstance(msg.body, (types.MsgDetailedInfo, types.MsgNewDetailedInfo)):
-                self.pending_acks.add(msg.body.answer_msg_id)
-                continue
-
-            if isinstance(msg.body, types.NewSessionCreated):
-                continue
-
-            msg_id = None
-
-            if isinstance(msg.body, (types.BadMsgNotification, types.BadServerSalt)):
-                msg_id = msg.body.bad_msg_id
-            elif isinstance(msg.body, (core.FutureSalts, types.RpcResult)):
-                msg_id = msg.body.req_msg_id
-            elif isinstance(msg.body, types.Pong):
-                msg_id = msg.body.msg_id
-            else:
-                if self.client is not None:
-                    self.client.updates_queue.put(msg.body)
-
-            if msg_id in self.results:
-                self.results[msg_id].value = getattr(msg.body, "result", msg.body)
-                self.results[msg_id].event.set()
-
-        if len(self.pending_acks) >= self.ACKS_THRESHOLD:
-            log.info("Send {} acks".format(len(self.pending_acks)))
-
-            try:
-                self._send(types.MsgsAck(list(self.pending_acks)), False)
-            except (OSError, TimeoutError):
-                pass
-            else:
-                self.pending_acks.clear()
-
     def ping(self):
         log.debug("PingThread started")
 
@@ -368,7 +375,7 @@ class Session:
 
         log.debug("RecvThread stopped")
 
-    def _send(self, data: Object, wait_response: bool = True):
+    def _send(self, data: Object, wait_response: bool = True, timeout: float = WAIT_TIMEOUT):
         message = self.msg_factory(data)
         msg_id = message.msg_id
 
@@ -384,7 +391,7 @@ class Session:
             raise e
 
         if wait_response:
-            self.results[msg_id].event.wait(self.WAIT_TIMEOUT)
+            self.results[msg_id].event.wait(timeout)
             result = self.results.pop(msg_id).value
 
             if result is None:
@@ -399,11 +406,11 @@ class Session:
             else:
                 return result
 
-    def send(self, data: Object, retries: int = MAX_RETRIES):
+    def send(self, data: Object, retries: int = MAX_RETRIES, timeout: float = WAIT_TIMEOUT):
         self.is_connected.wait(self.WAIT_TIMEOUT)
 
         try:
-            return self._send(data)
+            return self._send(data, timeout=timeout)
         except (OSError, TimeoutError, InternalServerError) as e:
             if retries == 0:
                 raise e from None
@@ -414,4 +421,4 @@ class Session:
                     datetime.now(), type(data)))
 
             time.sleep(0.5)
-            return self.send(data, retries - 1)
+            return self.send(data, retries - 1, timeout)
